/********************************************************
Workspace: IAR+SPLib
Chip type: STM8S003K3 (16MHz HSI)
Project  : si controller scr|hf convertor|five leds
Version  : 0
Date     : Nov 2017
*********************************************************/
#include "stm8s.h"
#include "iostm003.c"
#include <stdbool.h>
#include "bitset.c"
#include "defines.c"
#include "periph_init.c"
#include "eeprom_routines.c"
#include "scale_work.c"
#include "beep_work.c"
#include "adc_work.c"
#include "ledwork.c"
#include "kwork.c"
#include "tc_work.c"


volatile u16 tim0,tim1,tim2,tim3,tim4;
volatile bool iwdg_t4;


u16 scale_val;

void devcon();


extern "C"
{ 
 INTERRUPT_HANDLER(TIM4_OVF, 23)   //1000Hz
 {
  //adc_work
  if(!adc_ready)
  {
   static bool trg=0;
   
   if(!trg)
   {
    trg = 1;
    ADC1->CSR  = 1;
    ADC1->CR1 |= ADC1_CR1_ADON; 
   }
   else
   {
    trg = 0;
    adc_ready = 1;
    tin_retval = *((u16*)&ADC1->DB0RH); 
    ref_retval = *((u16*)&ADC1->DB1RH);    
   }
  }
  //end of adc work 

  if(tim0)tim0--;
  if(tim1)tim1--;
  if(tim2)tim2--;  
  if(tim3)tim3--;   
  if(tim4)tim4--;    
  
  if(beeptim)beeptim--;
  else BEEP_Cmd(DISABLE);
  if(adc_tim)adc_tim--;  
  if(lb_tim)lb_tim--;
 
  scan_keys();
  
  iwdg_t4=1;
  TIM4->SR1 = 0;
 }
}

void main()
{
 cfg.gpio();
 cfg.clk();
 cfg.adc(); 
 cfg.t4();
 cfg.ee();

 //cfg.iwdg();

 asm("rim");
 
m:
 {
  devcon();  
  display_scale(scale_val);  

  if(iwdg_t4){iwdg_t4=0;IWDG_ReloadCounter();}
 }
goto m;
}


void devcon()
{
 static bool itr=0;
 static enum {start,work,sdn,err}dev_st = start;
 static u16 TC;
 static bool btn_trg, btn_htrg;
 static u16 t_set=read_u16();

 
 if(btn[0]==rls)btn_trg=btn_htrg=0;
 else if(dev_st!=sdn && dev_st!=err)
 {
  if(btn[0]==prs && !btn_trg)  
  {  
   btn_trg=1;  
   
   if(t_set == 450)t_set = 250;   
   else if(t_set<450-50)t_set+=50;   
   else t_set = 450;   
      
   scale_val = t2scale(t_set,0);   
   tim2 = 2000;   
  } 
  else if(btn[0]==hold && !btn_htrg)
  {   
   if(!tim3)
   {
    tim3=300;
    if(t_set == 450)t_set = 250;  
    else if(t_set<450-50)t_set+=50;  
    else t_set = 450;  
    
    scale_val = t2scale(t_set,0);   
    tim2 = 2000;
   }
  }
 }
 
 if(!tim4)
 {
  tim4 = 0xffff/2;
  if(t_set!=read_u16())
  {
   //beep(1000, 4000);
   write_u16(t_set);
  }
 }
 
 switch (dev_st)
 {
  case start:
  {
   static u8 cntdn;
   static bool i_trg;
   if(!itr)
   {
    itr=1;
    scr_on;

    tim0 = 1000;
    tim1 = 0xffff;
    tim2 = 0;
    cntdn = 5;
    i_trg = 0;
    
    scale_val = bin(11111);    
    beep(200, 1000);    
    
    if(t_set>450 || t_set<250) t_set=300;
   } 
   
   
   if(!tim0)
   {  
    if(!i_trg)
    {
     i_trg = 1;
     tim0 = 1500;
     scale_val = t2scale(t_set,0);
    }
    else
    {
     if(!tim2)
     {
      u16 tmp;
       
      if(read_adc(&tmp)==1)
      {
       TC = U2TC(tmp);
       
       if(TC > t_set+50) dev_st = work; 
       else
       {
        tim2 = 1000;      
        scale_val = t2scale(TC,1);        
       }
      }
     } 
    }
   }
   
   if(!tim1)
   {
    tim1=0xffff;
    if(cntdn)cntdn--;
    else dev_st = err;    
   }
   
   if(dev_st!=start){itr=0;}
  }
  break;

  case work:
  {
   if(!itr)
   {
    itr=1;
    beep(200, 4000);
   } 
   
   //1s timer
   if(!tim2)    
   { 
    //adc work surveillance
    static bool adc_trg=0;
    if(!adc_trg)
    {
     adc_trg=1;
     tim0 = 2000;
    }
    if(!tim0)
    {
     scr_off;
     while(1)scale_val = bin(11111); //resetting by watchdog
    }
    //end of adc work surveillance
    
    //adc processing
    u16 tmp;
    
    if(read_adc(&tmp)==1) 
    { 
     static u16 tc_tmp; 
     s8 dir;      
      
     tim2 = 1000;
     
     TC = U2TC(tmp); 
    
     if(tc_tmp<TC)dir=1; 
     else if(tc_tmp>TC)dir=-1;  
     else dir = 0;  
      
     tc_tmp = TC;       
       
     scale_val = t2scale(TC,dir);       
     
     //t reg
     if(TC<t_set)scr_on; 
     else if(TC>t_set+10)scr_off; 
     //end of t reg
     
     //no heat err detection
     #define cntdn_val 10
     static u8 cntdn = cntdn_val;
     if(TC<100)
     {
      if(cntdn)cntdn--;
      else 
      {
       cntdn = cntdn_val;
       dev_st = err;      
      }
     }
     else if(cntdn<cntdn_val)cntdn++;
     //end of no heat err detection
     
     //overheat detection
     if(TC>500)beep(100,4000);
     //end of overheat detection

     //cutoff counter
     static u16 cutoff_cnt = 0x4000; //4,5Hrs
     
     if(cutoff_cnt)cutoff_cnt--;
     else dev_st = sdn;
     
     static u16 t_set_tmp;
     static bool t_set_beep_trg;
     
     if(t_set_tmp != t_set)
     {
      t_set_tmp = t_set;
      cutoff_cnt = 0x4000;
      t_set_beep_trg = 1;
     }
     
     if(t_set_beep_trg && TC>t_set && TC<t_set+10)
     {
      t_set_beep_trg = 0;
      beep(200,2000);
     }
     //end of cutoff counter  
     
     adc_trg=0; //adc work surveillance trigger reset
    } 
    //end of adc processing
   }     
   //end of 1s timer
 
   
   if(dev_st!=work){itr=0;}
  }
  break;
  
  case sdn:
  {
   if(!itr)
   {
    itr=1;
    scr_off;
    scale_val = bin(10000);
    beep(2000, 1000);
   }   
   
   if(!tim0)
   {
    tim0=1000;
    scale_val^= bin(10000); 
   }

   if(btn[0]==hold && !btn_htrg)
   {
    btn_htrg=1;
    dev_st = start;
   }
      

   if(dev_st!=sdn){itr=0;}
  }
  break; 

  case err:
  {
   if(!itr)
   {
    itr=1;
    scr_off;
    tim0 = 1000;
    scale_val = bin(10101);
   }   
   
   if(!tim0)
   {
    tim0=1000;
    scale_val^=bin(11111);  
    beep(200, 1000);
   }

   if(btn[0]==hold && !btn_htrg)
   {
    btn_htrg=1;
    dev_st = start;
   }
   
   if(dev_st!=err){itr=0;}
  }
  break;
  
  default:break;
 }

}

